#!/usr/bin/perl
use warnings;
use strict;
use Tie::File;
use Getopt::Long;	# Support GetOptions().
my $help = 0;		# True if user specifies -h option
my $uninstall = 0;	# True if user specifies -u option
my @vhContents;

if (!GetOptions('help' => \$help, 'uninstall' => \$uninstall)
	|| $help || @ARGV > 0)
{
	print "\nUsage: perl $0 [Options]\n\n".
	"    Adds statement to /etc/apache2/vhosts.d/vhost-ssl.conf\n".
	"    to includeOptional /etc/opt/novell/httpd/sslconf.d/*.conf\n".
	"    Options:\n".
	"    -u, --uninstall            Removes novell include statement\n".
	"    -h, --help                 display this help information\n";
	exit 1;
}

my $vhost_file = "/etc/apache2/vhosts.d/vhost-ssl.conf";
my $str_include = "Include /etc/opt/novell/httpd/sslconf.d/*.conf";
my $str_includeopt = "IncludeOptional /etc/opt/novell/httpd/sslconf.d/*.conf";
my $str_cipher = "SSLCipherSuite HIGH:!MEDIUM:!LOW:!SHA:!SHA256:!SHA284:!SHA384:!aNULL\@STRENGTH";

use Fcntl 'O_RDWR';
tie @vhContents, 'Tie::File', $vhost_file, mode => O_RDWR or die "\"$vhost_file\" file not found";

sub dbprint
{
   print STDERR @_ if $ENV{'DEBUG'};
}

my $virtualHostEnd = 0;
my $includeFound = 0;

dbprint "Checking vhost-ssl.conf\n";

if (findNovellInclude($str_include))
{
	if ($uninstall)
	{
		removeNovellInclude();
	}
}
else
{
	if (not $uninstall && !findNovellInclude($str_includeopt))
	{
		addNovellInclude();
	}
}

if (!findNovellInclude($str_cipher) && not $uninstall)
{
	addCipher();
}

findAndReplaceSSLProtocol();

# set the separator to ''
$"='';

sub findNovellInclude
{
   my $i;
   my ($str) = @_;
   $includeFound = 0;
   for ($i = 0; $i < scalar(@vhContents); $i++)
   {
      my $idx = index $vhContents[$i], $str;
      if ($idx ne -1)
      {
         $includeFound = $i;
      }

      $idx = index $vhContents[$i], "</VirtualHost>";
      if ($idx ne -1)
      {
         $virtualHostEnd = $i;
         return $includeFound
      }	
   }

   return $includeFound;
}

sub findAndReplaceSSLProtocol
{
   my $i;
   my $flag = 0;
 
   for ($i = 0; $i < scalar(@vhContents); $i++)
   {     
      my $SSLProtocolOccurance = index $vhContents[$i], "SSLProtocol all -SSLv2 SSLv3";
      my $commentOccurance = index $vhContents[$i], "#";
      if ($SSLProtocolOccurance ne -1 && $commentOccurance ne -1)
      {
         $flag = 1;
	 $i++;
      }
    
      if($flag eq 1)
      {
	 my $SSLProtocolAllOccurance = index $vhContents[$i], "SSLProtocol all";
         my $SSLVersionOccurance = index $vhContents[$i], "SSLv";
         my $TLSOccurance = index $vhContents[$i], "TLS";
         my $commentOccurance = index $vhContents[$i], "#";
	 my $negationOccurance = index $vhContents[$i], "-";

         if ($SSLProtocolAllOccurance ne -1 && $SSLVersionOccurance eq -1 && $TLSOccurance eq -1 && $commentOccurance eq -1 && $negationOccurance eq -1)
         {
            $vhContents[$i]="#".$vhContents[$i]."\n\n";
            $vhContents[$i].="\tSSLProtocol all -SSLv2 -SSLv3\n";
	    return;
         }
      }
   }
}

sub addNovellInclude
{
   splice @vhContents, $virtualHostEnd++, 0, "\tIncludeOptional /etc/opt/novell/httpd/sslconf.d/*.conf\n\n";
}

sub removeNovellInclude
{
	splice @vhContents, $includeFound, 1;
}

sub addCipher
{
	splice @vhContents, $virtualHostEnd++, 0, "\tSSLCipherSuite HIGH:!MEDIUM:!LOW:!SHA:!SHA256:!SHA284:!SHA384:!aNULL\@STRENGTH\n\n";
}
