#!/bin/sh
# Copyright (C) [2011] Novell, Inc.  All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED, PERFORMED,
# DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN
# CONSENT.  USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD
# SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
# INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK,
# AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
# OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE,
# ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER
# DEALINGS IN THE WORK.
#
# -----------------------------------------------------------------------------
# Script: novell-gw-config-move.sh
# This script copies the application files of GroupWise from Tomcat5 to Tomcat 6 and 
# makes necessary generic changes to tomcat for working of the products.
# Designed for move applications from GroupWise 8.x and Ascot versions:
#  GroupWise Web Access
#  GroupWise Calendar Publishing Host
#  GroupWise Monitor
# -----------------------------------------------------------------------------

CURRENT_TOMCAT_VERSION='tomcat'
OLDER_TOMCAT_VERSION='tomcat6'
TOMCAT_BACKUP='tomcat5_backup'
GROUPWISE_WEBACC='gw'
GROUPWISE_CALHOST='gwcal'
GROUPWISE_MONITOR='gwmon'
GROUPWISE_WEBACC_NAME='GroupWise_Web_Access'
GROUPWISE_CALHOST_NAME='GroupWise_Calendar_Publishing_Host'
GROUPWISE_MONITOR_NAME='GroupWise_Monitor'
GROUPWISE_WEBACC_WAR='gw.war'
GROUPWISE_CALHOST_WAR='gwcal.war'
GROUPWISE_MONITOR_WAR='gwmon.war'
GROUPWISE_WEBACC_WAR_LOCATION='webaccess'
GROUPWISE_CALHOST_WAR_LOCATION='calhost'
GROUPWISE_MONITOR_WAR_LOCATION='monitor'


function moveApplication()
{
	if [[ -d $1 ]];then
		echo "$2 found!!! Moving it to the $CURRENT_TOMCAT_VERSION Now........"

		cp -r ./$1 /var/opt/novell/$CURRENT_TOMCAT_VERSION/webapps
		cd /var/opt/novell/$CURRENT_TOMCAT_VERSION/webapps/$1

		echo "$2 has been Moved to "`pwd`

		ln -sf /opt/novell/groupwise/$4/$3 /var/opt/novell/$CURRENT_TOMCAT_VERSION/webapps/$3
		unlink /opt/novell/groupwise/$4/$1
		ln -sf /var/opt/novell/$CURRENT_TOMCAT_VERSION/webapps/$1 /opt/novell/groupwise/$4/$1

		echo "Config Move of $2 COMPLETED SUCCESSFULLY"
	else
		echo "No $2 Installed on current M/C"
	fi
}



cd /var/opt/novell

if [[ -d $CURRENT_TOMCAT_VERSION && -d $OLDER_TOMCAT_VERSION ]];then
	cd $OLDER_TOMCAT_VERSION/webapps

	moveApplication $GROUPWISE_WEBACC $GROUPWISE_WEBACC_NAME $GROUPWISE_WEBACC_WAR $GROUPWISE_WEBACC_WAR_LOCATION
	cd /var/opt/novell/$OLDER_TOMCAT_VERSION/webapps
	moveApplication $GROUPWISE_CALHOST $GROUPWISE_CALHOST_NAME $GROUPWISE_CALHOST_WAR $GROUPWISE_CALHOST_WAR_LOCATION
	cd /var/opt/novell/$OLDER_TOMCAT_VERSION/webapps
	moveApplication $GROUPWISE_MONITOR $GROUPWISE_MONITOR_NAME $GROUPWISE_MONITOR_WAR $GROUPWISE_MONITOR_WAR_LOCATION

	rcnovell-tomcat restart
	rcapache2 restart

	cd /var/opt/novell
	mv $OLDER_TOMCAT_VERSION $TOMCAT_BACKUP
	echo "The $OLDER_TOMCAT_VERSION is backed up as $TOMCAT_BACKUP at /var/opt/novell"
else 

	echo "Can't find $OLDER_TOMCAT_VERSION and/or $CURRENT_TOMCAT_VERSION directory/directories in /var/opt/novell"

fi
